@@ -133,6 +133,8 @@ try:
         roll,
         rot90,
         round,
+        searchsorted,
+        select,
         shape,
         squeeze,
         swapaxes,
