@@ -1,12 +1,6 @@
 import sys
 from distutils.version import LooseVersion
 
-# TODO: remove this import once dask requires distributed > 2.3.2
-from .utils import apply  # noqa
-
-# TODO: remove this once dask requires distributed >= 2.2.0
-unicode = str  # noqa
-
 try:
     from math import prod
 except ImportError:
