@@ -23,7 +23,7 @@ from .dispatch import (
     group_split_dispatch,
     hash_object_dispatch,
     is_categorical_dtype_dispatch,
-    make_meta,
+    make_meta_dispatch,
     make_meta_obj,
     meta_nonempty,
     tolist_dispatch,
@@ -57,12 +57,12 @@ def _(x):
     return x
 
 
-@make_meta.register((pd.Series, pd.DataFrame))
+@make_meta_dispatch.register((pd.Series, pd.DataFrame))
 def make_meta_pandas(x, index=None):
     return x.iloc[:0]
 
 
-@make_meta.register(pd.Index)
+@make_meta_dispatch.register(pd.Index)
 def make_meta_index(x, index=None):
     return x[0:0]
 
@@ -76,6 +76,12 @@ except ImportError:
     pass
 
 
+@meta_nonempty.register(pd.DatetimeTZDtype)
+@make_meta_dispatch.register(pd.DatetimeTZDtype)
+def make_meta_pandas_datetime_tz(x, index=None):
+    return _nonempty_scalar(x)
+
+
 @make_meta_obj.register(meta_object_types)
 def make_meta_object(x, index=None):
     """Create an empty pandas object containing the desired metadata.
@@ -109,7 +115,7 @@ def make_meta_object(x, index=None):
         return x[:0]
 
     if index is not None:
-        index = make_meta(index)
+        index = make_meta_dispatch(index)
 
     if isinstance(x, dict):
         return pd.DataFrame(
@@ -524,7 +530,7 @@ def is_categorical_dtype_pandas(obj):
 @group_split_dispatch.register_lazy("cudf")
 @get_parallel_type.register_lazy("cudf")
 @meta_nonempty.register_lazy("cudf")
-@make_meta.register_lazy("cudf")
+@make_meta_dispatch.register_lazy("cudf")
 @make_meta_obj.register_lazy("cudf")
 def _register_cudf():
     import dask_cudf  # noqa: F401
