@@ -1,4 +1,3 @@
-import copy
 from collections.abc import Mapping
 from io import BytesIO
 from warnings import catch_warnings, simplefilter, warn
@@ -72,11 +71,17 @@ class CSVFunctionWrapper:
         columns = [c for c in self.head.columns if c in columns]
         if columns == self.columns:
             return self
-        func = copy.deepcopy(self)
-        func.head = self.head[columns]
-        func.dtypes = {c: self.dtypes[c] for c in columns}
-        func.columns = columns
-        return func
+        return CSVFunctionWrapper(
+            self.full_columns,
+            columns,
+            self.colname,
+            self.head[columns],
+            self.header,
+            self.reader,
+            {c: self.dtypes[c] for c in columns},
+            self.enforce,
+            self.kwargs,
+        )
 
     def __call__(self, part):
 
