@@ -7,6 +7,7 @@ import pytest
 
 import dask.dataframe as dd
 from dask.dataframe import read_orc
+from dask.dataframe.optimize import optimize_dataframe_getitem
 from dask.dataframe.utils import assert_eq
 
 pytest.importorskip("pyarrow.orc")
@@ -64,6 +65,14 @@ def test_orc_single(orc_files):
     with pytest.raises(ValueError, match="nonexist"):
         read_orc(fn, columns=["time", "nonexist"])
 
+    # Check that `optimize_dataframe_getitem` changes the
+    # `columns` attribute of the "read-orc" layer
+    d3 = d[columns]
+    keys = [(d3._name, i) for i in range(d3.npartitions)]
+    graph = optimize_dataframe_getitem(d3.__dask_graph__(), keys)
+    key = [k for k in graph.layers.keys() if k.startswith("read-orc-")][0]
+    assert set(graph.layers[key].columns) == set(columns)
+
 
 def test_orc_multiple(orc_files):
     d = read_orc(orc_files[0])
